# WARNING: Please don't edit this file. It was generated by Python/WinRT v3.1.0

import winrt.system
from winrt._winrt_windows_ui_accessibility import (
    ScreenReaderPositionChangedEventArgs,
    ScreenReaderService,
)

__all__ = [
    "ScreenReaderPositionChangedEventArgs",
    "ScreenReaderService",
]

